@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-# Copyright (C) Duncan Macleod (2016)
+# Copyright (C) Duncan Macleod (2016-2019)
 #
 # This file is part of GWpy.
 #
@@ -35,13 +35,17 @@ import numpy
 from numpy import fft as npfft
 
 from ..segments import Segment
-from ..timeseries import TimeSeries
 
 __author__ = 'Duncan Macleod <duncan.macleod@ligo.org>'
 __credits__ = 'Scott Coughlin <scott.coughlin@ligo.org>, ' \
               'Alex Urban <alexander.urban@ligo.org>'
 __all__ = ['QTiling', 'QPlane', 'QTile', 'QGram', 'q_scan']
 
+# q-transform defaults
+DEFAULT_FRANGE = (0, float('inf'))
+DEFAULT_MISMATCH = 0.2
+DEFAULT_QRANGE = (4, 64)
+
 
 # -- object class definitions -------------------------------------------------
 
@@ -53,7 +57,7 @@ class QObject(object):
     """
     # pylint: disable=too-few-public-methods
 
-    def __init__(self, duration, sampling, mismatch=.2):
+    def __init__(self, duration, sampling, mismatch=DEFAULT_MISMATCH):
         self.duration = float(duration)
         self.sampling = float(sampling)
         self.mismatch = float(mismatch)
@@ -72,7 +76,7 @@ class QBase(QObject):
 
     This class just provides a property for Q-prime = Q / sqrt(11)
     """
-    def __init__(self, q, duration, sampling, mismatch=.2):
+    def __init__(self, q, duration, sampling, mismatch=DEFAULT_MISMATCH):
         super(QBase, self).__init__(duration, sampling, mismatch=mismatch)
         self.q = float(q)
 
@@ -107,7 +111,9 @@ class QTiling(QObject):
         maximum fractional mismatch between neighbouring tiles
     """
     def __init__(self, duration, sampling,
-                 qrange=(4, 64), frange=(0, numpy.inf), mismatch=.2):
+                 qrange=DEFAULT_QRANGE,
+                 frange=DEFAULT_FRANGE,
+                 mismatch=DEFAULT_MISMATCH):
         super(QTiling, self).__init__(duration, sampling, mismatch=mismatch)
         self.qrange = (float(qrange[0]), float(qrange[1]))
         self.frange = [float(frange[0]), float(frange[1])]
@@ -219,7 +225,8 @@ class QPlane(QBase):
     mismatch : `float`
         maximum fractional mismatch between neighbouring tiles
     """
-    def __init__(self, q, frange, duration, sampling, mismatch=.2):
+    def __init__(self, q, frange, duration, sampling,
+                 mismatch=DEFAULT_MISMATCH):
         super(QPlane, self).__init__(q, duration, sampling, mismatch=mismatch)
         self.frange = [float(frange[0]), float(frange[1])]
 
@@ -320,7 +327,8 @@ class QPlane(QBase):
 class QTile(QBase):
     """Representation of a tile with fixed Q and frequency
     """
-    def __init__(self, q, frequency, duration, sampling, mismatch=.2):
+    def __init__(self, q, frequency, duration, sampling,
+                 mismatch=DEFAULT_MISMATCH):
         super(QTile, self).__init__(q, duration, sampling, mismatch=mismatch)
         self.frequency = frequency
 
@@ -408,6 +416,8 @@ class QTile(QBase):
             a `TimeSeries` of the energy from the Q-transform of
             this tile against the data.
         """
+        from ..timeseries import TimeSeries
+
         windowed = fseries[self.get_data_indices()] * self.get_window()
         # pad data, move negative frequencies to the end, and IFFT
         padded = numpy.pad(windowed, self.padding, mode='constant')
@@ -425,11 +435,12 @@ class QTile(QBase):
         if norm:
             norm = norm.lower() if isinstance(norm, string_types) else norm
             if norm in (True, 'median'):
-                return energy / energy.median()
+                narray = energy / energy.median()
             elif norm in ('mean',):
-                return energy / energy.mean()
+                narray = energy / energy.mean()
             else:
                 raise ValueError("Invalid normalisation %r" % norm)
+            return narray.astype("float32", casting="same_kind", copy=False)
         return energy
 
 
@@ -468,26 +479,29 @@ class QGram(object):
                 })
         return peak
 
-    def interpolate(self, tres=.001, fres=.5, logf=False, outseg=None):
+    def interpolate(self, tres="<default>", fres="<default>", logf=False,
+                    outseg=None):
         """Interpolate this `QGram` over a regularly-gridded spectrogram
 
         Parameters
         ----------
         tres : `float`, optional
-            desired time resolution (seconds) of output `Spectrogram`
+            desired time resolution (seconds) of output `Spectrogram`,
+            default is `abs(outseg) / 1000.`
 
         fres : `float`, `int`, `None`, optional
             desired frequency resolution (Hertz) of output `Spectrogram`,
-            give `None` to skip this step and return the original resolution
+            or, if ``logf=True``, the number of frequency samples;
+            give `None` to skip this step and return the original resolution,
+            default is 0.5 Hz or 500 frequency samples
 
         logf : `bool`, optional
             boolean switch to enable (`True`) or disable (`False`) use of
-            log-sampled frequencies in the output `Spectrogram`,
-            if `True` then `fres` is interpreted as number of frequency
-            samples, default: `False`
+            log-sampled frequencies in the output `Spectrogram`
 
         outseg : `~gwpy.segments.Segment`, optional
-            GPS `[start, stop)` segment for output `Spectrogram`
+            GPS `[start, stop)` segment for output `Spectrogram`,
+            default is the full duration of the input
 
         Returns
         -------
@@ -504,6 +518,9 @@ class QGram(object):
 
         Notes
         -----
+        This method will return a `Spectrogram` of dtype ``float32`` if
+        ``norm`` is given, and ``float64`` otherwise.
+
         To optimize plot rendering with `~matplotlib.axes.Axes.pcolormesh`,
         the output `~gwpy.spectrogram.Spectrogram` can be given a log-sampled
         frequency axis by passing `logf=True` at runtime. The `fres` argument
@@ -521,6 +538,8 @@ class QGram(object):
         dtype = self.energies[0].dtype
         # build regular Spectrogram from peak-Q data by interpolating each
         # (Q, frequency) `TimeSeries` to have the same time resolution
+        if tres == "<default>":
+            tres = abs(Segment(outseg)) / 1000.
         xout = numpy.arange(*outseg, step=tres)
         nx = xout.size
         ny = frequencies.size
@@ -533,7 +552,8 @@ class QGram(object):
             xrow = numpy.arange(row.x0.value, (row.x0 + row.duration).value,
                                 row.dx.value)
             interp = InterpolatedUnivariateSpline(xrow, row.value)
-            out[:, i] = interp(xout)
+            out[:, i] = interp(xout).astype(dtype, casting="same_kind",
+                                            copy=False)
         if fres is None:
             return out
         # interpolate the spectrogram to increase its frequency resolution
@@ -541,16 +561,24 @@ class QGram(object):
         #     since they don't support log scaling
         interp = interp2d(xout, frequencies, out.value.T, kind='cubic')
         if not logf:
+            if fres == "<default>":
+                fres = .5
             outfreq = numpy.arange(
-                self.plane.frange[0], self.plane.frange[1], fres)
+                self.plane.frange[0], self.plane.frange[1], fres,
+                dtype=dtype)
         else:
+            if fres == "<default>":
+                fres = 500
             # using `~numpy.logspace` here to support numpy-1.7.1 for EPEL7,
             # but numpy-1.12.0 introduced the function `~numpy.geomspace`
             logfmin = numpy.log10(self.plane.frange[0])
             logfmax = numpy.log10(self.plane.frange[1])
-            outfreq = numpy.logspace(logfmin, logfmax, fres)
-        new = type(out)(interp(xout, outfreq).T,
-                        t0=outseg[0], dt=tres, frequencies=outfreq)
+            outfreq = numpy.logspace(logfmin, logfmax, num=int(fres))
+        new = type(out)(
+            interp(xout, outfreq).T.astype(
+                dtype, casting="same_kind", copy=False),
+            t0=outseg[0], dt=tres, frequencies=outfreq,
+        )
         new.q = self.plane.q
         return new
 
@@ -604,8 +632,9 @@ def next_power_of_two(x):
     return 2**(ceil(log(x, 2)))
 
 
-def q_scan(data, mismatch=0.2, qrange=(4, 64), frange=(0, float('inf')),
-           duration=None, sampling=None, **kwargs):
+def q_scan(data, mismatch=DEFAULT_MISMATCH, qrange=DEFAULT_QRANGE,
+           frange=DEFAULT_FRANGE, duration=None, sampling=None,
+           **kwargs):
     """Transform data by scanning over a `QTiling`
 
     This utility is provided mainly to allow direct manipulation of the
@@ -618,8 +647,7 @@ def q_scan(data, mismatch=0.2, qrange=(4, 64), frange=(0, float('inf')),
         the time- or frequency-domain input data
 
     mismatch : `float`, optional
-        maximum allowed fractional mismatch between neighbouring tiles,
-        default: 0.2
+        maximum allowed fractional mismatch between neighbouring tiles
 
     qrange : `tuple` of `float`, optional
         `(low, high)` range of Qs to scan
@@ -646,6 +674,7 @@ def q_scan(data, mismatch=0.2, qrange=(4, 64), frange=(0, float('inf')),
         expected false alarm rate (Hertz) of white Gaussian noise with the
         same peak energy and total duration as `qgram`
     """
+    from gwpy.timeseries import TimeSeries
     # prepare input
     if isinstance(data, TimeSeries):
         duration = abs(data.span)
